const $path = require("path");
const ts = require("typescript");
const glob = require("glob");

// Expression Transformer
const {parseExpressions} = require("../expression-transformer");

// TypeScript config file
const options = require($path.join(process.cwd(), "tsconfig"));

console.log("Compiling by", $path.join(process.cwd(), "tsconfig.json"));
console.log(options);

// options.include must be set; **/*.ts otherwise
if (!options.hasOwnProperty("include")) {
	options.include = ["./**/*.ts"];
}

/**
 * Gets paths from patterns
 * @param pattern
 * @param files
 * @returns {Promise<void>}
 */
async function getPathsFromPatterns(pattern, files) {
	return new Promise((resolve, reject) => {
		glob(pattern, {absolute: true}, async (err, filePaths) => {
			const promises = [];

			try {
				for (let path of filePaths) {
					if (files.indexOf(path) === -1) {
						files.push(path);
					}
				}
				await Promise.all(promises);
				resolve();
			} catch (ex) {
				reject(ex);
			}
		});
	});
}

/**
 * Returns files paths and contents
 * @param options
 * @returns {Promise<Array<{path: string, content: string}>>}
 */
async function getFiles(options) {
	const promises = [];
	const files = [];

	for (let pattern of options.include) {
		promises.push(
			getPathsFromPatterns(pattern, files)
		);

	}

	await Promise.all(promises);
	return Promise.resolve(files);
}

/**
 * Compiled typescript files with transformer by tsconfig.json from calling directory
 * @returns {Promise<void>}
 */
exports.compile = async function compile() {
	try {
		const promises = [];
		const files = await getFiles(options);
		const program = ts.createProgram(files, options.compilerOptions);
		program.emit(undefined, undefined, undefined, false, {before: [parseExpressions(program)]});

		await Promise.all(promises);
	} catch (e) {
		console.error(e);
	}
}